@@ -19,7 +19,7 @@
  */
 
 /**
- * This is a simple example in order to show how 802.11n compressed block ack mechanism could be used.
+ * This is a simple example in order to show how 802.11e compressed block ack mechanism could be used.
  *
  * Network topology:
  * 
@@ -62,25 +62,27 @@ int main (int argc, char const* argv[])
   YansWifiPhyHelper phy = YansWifiPhyHelper::Default ();
   phy.SetChannel (channel.Create ());
 
-  WifiHelper wifi = WifiHelper::Default ();
-  QosWifiMacHelper mac = QosWifiMacHelper::Default ();
+  WifiHelper wifi;
+  WifiMacHelper mac;
   /* disable fragmentation */
   wifi.SetRemoteStationManager ("ns3::AarfWifiManager", "FragmentationThreshold", UintegerValue (2500));
 
   Ssid ssid ("My-network");
 
   mac.SetType ("ns3::StaWifiMac",
+               "QosSupported", BooleanValue (true),
                "Ssid", SsidValue (ssid),
-               "ActiveProbing", BooleanValue (false));
+               "ActiveProbing", BooleanValue (false),
   /* setting blockack threshold for sta's BE queue */
-  mac.SetBlockAckThresholdForAc (AC_BE, 2);
-  /* setting block inactivity timeout to 3*1024 = 3072 microseconds */ 
-  //mac.SetBlockAckInactivityTimeoutForAc (AC_BE, 3);
+               "BE_BlockAckThreshold", UintegerValue (2),
+  /* setting block inactivity timeout to 3*1024 = 3072 microseconds */
+               "BE_BlockAckInactivityTimeout", UintegerValue (3));
   NetDeviceContainer staDevice = wifi.Install (phy, mac, sta);
 
   mac.SetType ("ns3::ApWifiMac",
-               "Ssid", SsidValue (ssid));
-  mac.SetBlockAckThresholdForAc (AC_BE, 0);
+               "QosSupported", BooleanValue (true),
+               "Ssid", SsidValue (ssid),
+               "BE_BlockAckThreshold", UintegerValue (0));
   NetDeviceContainer apDevice = wifi.Install (phy, mac, ap);
 
   /* Setting mobility model */
@@ -121,8 +123,8 @@ int main (int argc, char const* argv[])
   DataRate dataRate ("1Mb/s");
   OnOffHelper onOff ("ns3::UdpSocketFactory", Address (InetSocketAddress (apIf.GetAddress (0), port)));
   onOff.SetAttribute ("DataRate", DataRateValue (dataRate));
-  onOff.SetAttribute ("OnTime", RandomVariableValue (ConstantVariable (0.01)));
-  onOff.SetAttribute ("OffTime", RandomVariableValue (ConstantVariable (8)));
+  onOff.SetAttribute ("OnTime", StringValue ("ns3::ConstantRandomVariable[Constant=0.01]"));
+  onOff.SetAttribute ("OffTime", StringValue ("ns3::ConstantRandomVariable[Constant=8]"));
   onOff.SetAttribute ("PacketSize", UintegerValue (50));
 
   ApplicationContainer staApps = onOff.Install (sta);
