@@ -28,12 +28,22 @@
 #include "assert.h"
 #include "log.h"
 
-#include <math.h>
+#include <cmath>
 
-NS_LOG_COMPONENT_DEFINE ("DefaultSimulatorImpl");
+
+/**
+ * \file
+ * \ingroup simulator
+ * Implementation of class ns3::DefaultSimulatorImpl.
+ */
 
 namespace ns3 {
 
+// Note:  Logging in this file is largely avoided due to the
+// number of calls that are made to these functions and the possibility
+// of causing recursions leading to stack overflow
+NS_LOG_COMPONENT_DEFINE ("DefaultSimulatorImpl");
+
 NS_OBJECT_ENSURE_REGISTERED (DefaultSimulatorImpl);
 
 TypeId
@@ -41,6 +51,7 @@ DefaultSimulatorImpl::GetTypeId (void)
 {
   static TypeId tid = TypeId ("ns3::DefaultSimulatorImpl")
     .SetParent<SimulatorImpl> ()
+    .SetGroupName ("Core")
     .AddConstructor<DefaultSimulatorImpl> ()
   ;
   return tid;
@@ -48,6 +59,7 @@ DefaultSimulatorImpl::GetTypeId (void)
 
 DefaultSimulatorImpl::DefaultSimulatorImpl ()
 {
+  NS_LOG_FUNCTION (this);
   m_stop = false;
   // uids are allocated from 4.
   // uid 0 is "invalid" events
@@ -59,15 +71,21 @@ DefaultSimulatorImpl::DefaultSimulatorImpl ()
   m_currentTs = 0;
   m_currentContext = 0xffffffff;
   m_unscheduledEvents = 0;
+  m_eventsWithContextEmpty = true;
+  m_main = SystemThread::Self();
 }
 
 DefaultSimulatorImpl::~DefaultSimulatorImpl ()
 {
+  NS_LOG_FUNCTION (this);
 }
 
 void
 DefaultSimulatorImpl::DoDispose (void)
 {
+  NS_LOG_FUNCTION (this);
+  ProcessEventsWithContext ();
+
   while (!m_events->IsEmpty ())
     {
       Scheduler::Event next = m_events->RemoveNext ();
@@ -79,6 +97,7 @@ DefaultSimulatorImpl::DoDispose (void)
 void
 DefaultSimulatorImpl::Destroy ()
 {
+  NS_LOG_FUNCTION (this);
   while (!m_destroyEvents.empty ()) 
     {
       Ptr<EventImpl> ev = m_destroyEvents.front ().PeekEventImpl ();
@@ -94,6 +113,7 @@ DefaultSimulatorImpl::Destroy ()
 void
 DefaultSimulatorImpl::SetScheduler (ObjectFactory schedulerFactory)
 {
+  NS_LOG_FUNCTION (this << schedulerFactory);
   Ptr<Scheduler> scheduler = schedulerFactory.Create<Scheduler> ();
 
   if (m_events != 0)
@@ -128,6 +148,8 @@ DefaultSimulatorImpl::ProcessOneEvent (void)
   m_currentUid = next.key.m_uid;
   next.impl->Invoke ();
   next.impl->Unref ();
+
+  ProcessEventsWithContext ();
 }
 
 bool 
@@ -136,24 +158,45 @@ DefaultSimulatorImpl::IsFinished (void) const
   return m_events->IsEmpty () || m_stop;
 }
 
-uint64_t
-DefaultSimulatorImpl::NextTs (void) const
+void
+DefaultSimulatorImpl::ProcessEventsWithContext (void)
 {
-  NS_ASSERT (!m_events->IsEmpty ());
-  Scheduler::Event ev = m_events->PeekNext ();
-  return ev.key.m_ts;
-}
+  if (m_eventsWithContextEmpty)
+    {
+      return;
+    }
 
-Time
-DefaultSimulatorImpl::Next (void) const
-{
-  return TimeStep (NextTs ());
+  // swap queues
+  EventsWithContext eventsWithContext;
+  {
+    CriticalSection cs (m_eventsWithContextMutex);
+    m_eventsWithContext.swap(eventsWithContext);
+    m_eventsWithContextEmpty = true;
+  }
+  while (!eventsWithContext.empty ())
+    {
+       EventWithContext event = eventsWithContext.front ();
+       eventsWithContext.pop_front ();
+       Scheduler::Event ev;
+       ev.impl = event.event;
+       ev.key.m_ts = m_currentTs + event.timestamp;
+       ev.key.m_context = event.context;
+       ev.key.m_uid = m_uid;
+       m_uid++;
+       m_unscheduledEvents++;
+       m_events->Insert (ev);
+    }
 }
 
 void
 DefaultSimulatorImpl::Run (void)
 {
+  NS_LOG_FUNCTION (this);
+  // Set the current threadId as the main threadId
+  m_main = SystemThread::Self();
+  ProcessEventsWithContext ();
   m_stop = false;
+
   while (!m_events->IsEmpty () && !m_stop) 
     {
       ProcessOneEvent ();
@@ -164,31 +207,30 @@ DefaultSimulatorImpl::Run (void)
   NS_ASSERT (!m_events->IsEmpty () || m_unscheduledEvents == 0);
 }
 
-void
-DefaultSimulatorImpl::RunOneEvent (void)
-{
-  ProcessOneEvent ();
-}
-
 void 
 DefaultSimulatorImpl::Stop (void)
 {
+  NS_LOG_FUNCTION (this);
   m_stop = true;
 }
 
 void 
-DefaultSimulatorImpl::Stop (Time const &time)
+DefaultSimulatorImpl::Stop (Time const &delay)
 {
-  Simulator::Schedule (time, &Simulator::Stop);
+  NS_LOG_FUNCTION (this << delay.GetTimeStep ());
+  Simulator::Schedule (delay, &Simulator::Stop);
 }
 
 //
 // Schedule an event for a _relative_ time in the future.
 //
 EventId
-DefaultSimulatorImpl::Schedule (Time const &time, EventImpl *event)
+DefaultSimulatorImpl::Schedule (Time const &delay, EventImpl *event)
 {
-  Time tAbsolute = time + TimeStep (m_currentTs);
+  NS_LOG_FUNCTION (this << delay.GetTimeStep () << event);
+  NS_ASSERT_MSG (SystemThread::Equals (m_main), "Simulator::Schedule Thread-unsafe invocation!");
+
+  Time tAbsolute = delay + TimeStep (m_currentTs);
 
   NS_ASSERT (tAbsolute.IsPositive ());
   NS_ASSERT (tAbsolute >= TimeStep (m_currentTs));
@@ -204,23 +246,42 @@ DefaultSimulatorImpl::Schedule (Time const &time, EventImpl *event)
 }
 
 void
-DefaultSimulatorImpl::ScheduleWithContext (uint32_t context, Time const &time, EventImpl *event)
+DefaultSimulatorImpl::ScheduleWithContext (uint32_t context, Time const &delay, EventImpl *event)
 {
-  NS_LOG_FUNCTION (this << context << time.GetTimeStep () << m_currentTs << event);
+  NS_LOG_FUNCTION (this << context << delay.GetTimeStep () << event);
 
-  Scheduler::Event ev;
-  ev.impl = event;
-  ev.key.m_ts = m_currentTs + time.GetTimeStep ();
-  ev.key.m_context = context;
-  ev.key.m_uid = m_uid;
-  m_uid++;
-  m_unscheduledEvents++;
-  m_events->Insert (ev);
+  if (SystemThread::Equals (m_main))
+    {
+      Time tAbsolute = delay + TimeStep (m_currentTs);
+      Scheduler::Event ev;
+      ev.impl = event;
+      ev.key.m_ts = (uint64_t) tAbsolute.GetTimeStep ();
+      ev.key.m_context = context;
+      ev.key.m_uid = m_uid;
+      m_uid++;
+      m_unscheduledEvents++;
+      m_events->Insert (ev);
+    }
+  else
+    {
+      EventWithContext ev;
+      ev.context = context;
+      // Current time added in ProcessEventsWithContext()
+      ev.timestamp = delay.GetTimeStep ();
+      ev.event = event;
+      {
+        CriticalSection cs (m_eventsWithContextMutex);
+        m_eventsWithContext.push_back(ev);
+        m_eventsWithContextEmpty = false;
+      }
+    }
 }
 
 EventId
 DefaultSimulatorImpl::ScheduleNow (EventImpl *event)
 {
+  NS_ASSERT_MSG (SystemThread::Equals (m_main), "Simulator::ScheduleNow Thread-unsafe invocation!");
+
   Scheduler::Event ev;
   ev.impl = event;
   ev.key.m_ts = m_currentTs;
@@ -235,6 +296,8 @@ DefaultSimulatorImpl::ScheduleNow (EventImpl *event)
 EventId
 DefaultSimulatorImpl::ScheduleDestroy (EventImpl *event)
 {
+  NS_ASSERT_MSG (SystemThread::Equals (m_main), "Simulator::ScheduleDestroy Thread-unsafe invocation!");
+
   EventId id (Ptr<EventImpl> (event, false), m_currentTs, 0xffffffff, 2);
   m_destroyEvents.push_back (id);
   m_uid++;
@@ -244,6 +307,7 @@ DefaultSimulatorImpl::ScheduleDestroy (EventImpl *event)
 Time
 DefaultSimulatorImpl::Now (void) const
 {
+  // Do not add function logging here, to avoid stack overflow
   return TimeStep (m_currentTs);
 }
 
@@ -303,30 +367,30 @@ DefaultSimulatorImpl::Cancel (const EventId &id)
 }
 
 bool
-DefaultSimulatorImpl::IsExpired (const EventId &ev) const
+DefaultSimulatorImpl::IsExpired (const EventId &id) const
 {
-  if (ev.GetUid () == 2)
+  if (id.GetUid () == 2)
     {
-      if (ev.PeekEventImpl () == 0 ||
-          ev.PeekEventImpl ()->IsCancelled ())
+      if (id.PeekEventImpl () == 0 ||
+          id.PeekEventImpl ()->IsCancelled ())
         {
           return true;
         }
       // destroy events.
       for (DestroyEvents::const_iterator i = m_destroyEvents.begin (); i != m_destroyEvents.end (); i++)
         {
-          if (*i == ev)
+          if (*i == id)
             {
               return false;
             }
         }
       return true;
     }
-  if (ev.PeekEventImpl () == 0 ||
-      ev.GetTs () < m_currentTs ||
-      (ev.GetTs () == m_currentTs &&
-       ev.GetUid () <= m_currentUid) ||
-      ev.PeekEventImpl ()->IsCancelled ()) 
+  if (id.PeekEventImpl () == 0 ||
+      id.GetTs () < m_currentTs ||
+      (id.GetTs () == m_currentTs &&
+       id.GetUid () <= m_currentUid) ||
+      id.PeekEventImpl ()->IsCancelled ()) 
     {
       return true;
     }
@@ -339,8 +403,8 @@ DefaultSimulatorImpl::IsExpired (const EventId &ev) const
 Time 
 DefaultSimulatorImpl::GetMaximumSimulationTime (void) const
 {
-  // XXX: I am fairly certain other compilers use other non-standard
-  // post-fixes to indicate 64 bit constants.
+  /// \todo I am fairly certain other compilers use other non-standard
+  /// post-fixes to indicate 64 bit constants.
   return TimeStep (0x7fffffffffffffffLL);
 }
 
@@ -351,5 +415,3 @@ DefaultSimulatorImpl::GetContext (void) const
 }
 
 } // namespace ns3
-
-
