@@ -33,12 +33,22 @@
 #include "enum.h"
 
 
-#include <math.h>
+#include <cmath>
 
-NS_LOG_COMPONENT_DEFINE ("RealtimeSimulatorImpl");
+
+/**
+ * \file
+ * \ingroup realtime
+ * ns3::RealTimeSimulatorImpl implementation.
+ */
 
 namespace ns3 {
 
+// Note:  Logging in this file is largely avoided due to the
+// number of calls that are made to these functions and the possibility
+// of causing recursions leading to stack overflow
+NS_LOG_COMPONENT_DEFINE ("RealtimeSimulatorImpl");
+
 NS_OBJECT_ENSURE_REGISTERED (RealtimeSimulatorImpl);
 
 TypeId
@@ -46,6 +56,7 @@ RealtimeSimulatorImpl::GetTypeId (void)
 {
   static TypeId tid = TypeId ("ns3::RealtimeSimulatorImpl")
     .SetParent<SimulatorImpl> ()
+    .SetGroupName ("Core")
     .AddConstructor<RealtimeSimulatorImpl> ()
     .AddAttribute ("SynchronizationMode", 
                    "What to do if the simulation cannot keep up with real time.",
@@ -65,7 +76,7 @@ RealtimeSimulatorImpl::GetTypeId (void)
 
 RealtimeSimulatorImpl::RealtimeSimulatorImpl ()
 {
-  NS_LOG_FUNCTION_NOARGS ();
+  NS_LOG_FUNCTION (this);
 
   m_stop = false;
   m_running = false;
@@ -80,6 +91,8 @@ RealtimeSimulatorImpl::RealtimeSimulatorImpl ()
   m_currentContext = 0xffffffff;
   m_unscheduledEvents = 0;
 
+  m_main = SystemThread::Self();
+
   // Be very careful not to do anything that would cause a change or assignment
   // of the underlying reference counts of m_synchronizer or you will be sorry.
   m_synchronizer = CreateObject<WallClockSynchronizer> ();
@@ -87,13 +100,14 @@ RealtimeSimulatorImpl::RealtimeSimulatorImpl ()
 
 RealtimeSimulatorImpl::~RealtimeSimulatorImpl ()
 {
+  NS_LOG_FUNCTION (this);
 }
 
 void
 RealtimeSimulatorImpl::DoDispose (void)
 {
-  NS_LOG_FUNCTION_NOARGS ();
-  while (m_events->IsEmpty () == false)
+  NS_LOG_FUNCTION (this);
+  while (!m_events->IsEmpty ())
     {
       Scheduler::Event next = m_events->RemoveNext ();
       next.impl->Unref ();
@@ -106,7 +120,7 @@ RealtimeSimulatorImpl::DoDispose (void)
 void
 RealtimeSimulatorImpl::Destroy ()
 {
-  NS_LOG_FUNCTION_NOARGS ();
+  NS_LOG_FUNCTION (this);
 
   //
   // This function is only called with the private version "disconnected" from
@@ -131,7 +145,7 @@ RealtimeSimulatorImpl::Destroy ()
 void
 RealtimeSimulatorImpl::SetScheduler (ObjectFactory schedulerFactory)
 {
-  NS_LOG_FUNCTION_NOARGS ();
+  NS_LOG_FUNCTION (this << schedulerFactory);
 
   Ptr<Scheduler> scheduler = schedulerFactory.Create<Scheduler> ();
 
@@ -153,7 +167,6 @@ RealtimeSimulatorImpl::SetScheduler (ObjectFactory schedulerFactory)
 void
 RealtimeSimulatorImpl::ProcessOneEvent (void)
 {
-  NS_LOG_FUNCTION_NOARGS ();
   //
   // The idea here is to wait until the next event comes due.  In the case of
   // a realtime simulation, we want real time to be consumed between events.
@@ -383,7 +396,6 @@ RealtimeSimulatorImpl::ProcessOneEvent (void)
 bool 
 RealtimeSimulatorImpl::IsFinished (void) const
 {
-  NS_LOG_FUNCTION_NOARGS ();
   bool rc;
   {
     CriticalSection cs (m_mutex);
@@ -399,56 +411,55 @@ RealtimeSimulatorImpl::IsFinished (void) const
 uint64_t
 RealtimeSimulatorImpl::NextTs (void) const
 {
-  NS_LOG_FUNCTION_NOARGS ();
   NS_ASSERT_MSG (m_events->IsEmpty () == false, 
                  "RealtimeSimulatorImpl::NextTs(): event queue is empty");
   Scheduler::Event ev = m_events->PeekNext ();
   return ev.key.m_ts;
 }
 
-//
-// Calls NextTs().  Should be called with critical section locked.
-//
-Time
-RealtimeSimulatorImpl::Next (void) const
-{
-  NS_LOG_FUNCTION_NOARGS ();
-  return TimeStep (NextTs ());
-}
-
 void
 RealtimeSimulatorImpl::Run (void)
 {
-  NS_LOG_FUNCTION_NOARGS ();
+  NS_LOG_FUNCTION (this);
 
   NS_ASSERT_MSG (m_running == false, 
                  "RealtimeSimulatorImpl::Run(): Simulator already running");
 
+  // Set the current threadId as the main threadId
+  m_main = SystemThread::Self();
+
   m_stop = false;
   m_running = true;
   m_synchronizer->SetOrigin (m_currentTs);
 
-  for (;;) 
+  // Sleep until signalled
+  uint64_t tsNow;
+  uint64_t tsDelay = 1000000000; // wait time of 1 second (in nanoseconds)
+ 
+  while (!m_stop) 
     {
-      bool done = false;
-
+      bool process = false;
       {
         CriticalSection cs (m_mutex);
-        //
-        // In all cases we stop when the event list is empty.  If you are doing a 
-        // realtime simulation and you want it to extend out for some time, you must
-        // call StopAt.  In the realtime case, this will stick a placeholder event out
-        // at the end of time.
-        //
-        if (m_stop || m_events->IsEmpty ())
+
+        if (!m_events->IsEmpty ())
           {
-            done = true;
+            process = true;
+          }
+        else
+          {
+            // Get current timestamp while holding the critical section
+            tsNow = m_synchronizer->GetCurrentRealtime ();
           }
       }
-
-      if (done)
+ 
+      if (!process)
         {
-          break;
+          // Sleep until signalled
+          tsNow = m_synchronizer->Synchronize (tsNow, tsDelay);
+
+          // Re-check event queue
+          continue;
         }
 
       ProcessOneEvent ();
@@ -471,81 +482,36 @@ RealtimeSimulatorImpl::Run (void)
 bool
 RealtimeSimulatorImpl::Running (void) const
 {
-  NS_LOG_FUNCTION_NOARGS ();
   return m_running;
 }
 
 bool
 RealtimeSimulatorImpl::Realtime (void) const
 {
-  NS_LOG_FUNCTION_NOARGS ();
   return m_synchronizer->Realtime ();
 }
 
-//
-// This will run the first event on the queue without considering any realtime
-// synchronization.  It's mainly implemented to allow simulations requiring
-// the multithreaded ScheduleRealtimeNow() functions the possibility of driving
-// the simulation from their own event loop.
-//
-// It is expected that if there are any realtime requirements, the responsibility
-// for synchronizing with real time in an external event loop will be picked up
-// by that loop.  For example, they may call Simulator::Next() to find the 
-// execution time of the next event and wait for that time somehow -- then call
-// RunOneEvent to fire the event.
-// 
-void
-RealtimeSimulatorImpl::RunOneEvent (void)
-{
-  NS_LOG_FUNCTION_NOARGS ();
-
-  NS_ASSERT_MSG (m_running == false, 
-                 "RealtimeSimulatorImpl::RunOneEvent(): An internal simulator event loop is running");
-
-  EventImpl *event = 0;
-
-  //
-  // Run this in a critical section in case there's another thread around that
-  // may be inserting things onto the event list.
-  //
-  {
-    CriticalSection cs (m_mutex);
-
-    Scheduler::Event next = m_events->RemoveNext ();
-
-    NS_ASSERT (next.key.m_ts >= m_currentTs);
-    m_unscheduledEvents--;
-
-    NS_LOG_LOGIC ("handle " << next.key.m_ts);
-    m_currentTs = next.key.m_ts;
-    m_currentContext = next.key.m_context;
-    m_currentUid = next.key.m_uid;
-    event = next.impl;
-  }
-  event->Invoke ();
-  event->Unref ();
-}
-
 void 
 RealtimeSimulatorImpl::Stop (void)
 {
-  NS_LOG_FUNCTION_NOARGS ();
+  NS_LOG_FUNCTION (this);
   m_stop = true;
 }
 
 void 
-RealtimeSimulatorImpl::Stop (Time const &time)
+RealtimeSimulatorImpl::Stop (Time const &delay)
 {
-  Simulator::Schedule (time, &Simulator::Stop);
+  NS_LOG_FUNCTION (this << delay);
+  Simulator::Schedule (delay, &Simulator::Stop);
 }
 
 //
 // Schedule an event for a _relative_ time in the future.
 //
 EventId
-RealtimeSimulatorImpl::Schedule (Time const &time, EventImpl *impl)
+RealtimeSimulatorImpl::Schedule (Time const &delay, EventImpl *impl)
 {
-  NS_LOG_FUNCTION (time << impl);
+  NS_LOG_FUNCTION (this << delay << impl);
 
   Scheduler::Event ev;
   {
@@ -556,7 +522,7 @@ RealtimeSimulatorImpl::Schedule (Time const &time, EventImpl *impl)
     // multi-threaded, we need this calculation to be atomic.  You can see it is
     // here since we are running in a CriticalSection.
     //
-    Time tAbsolute = Simulator::Now () + time;
+    Time tAbsolute = Simulator::Now () + delay;
     NS_ASSERT_MSG (tAbsolute.IsPositive (), "RealtimeSimulatorImpl::Schedule(): Negative time");
     NS_ASSERT_MSG (tAbsolute >= TimeStep (m_currentTs), "RealtimeSimulatorImpl::Schedule(): time < m_currentTs");
     ev.impl = impl;
@@ -573,15 +539,28 @@ RealtimeSimulatorImpl::Schedule (Time const &time, EventImpl *impl)
 }
 
 void
-RealtimeSimulatorImpl::ScheduleWithContext (uint32_t context, Time const &time, EventImpl *impl)
+RealtimeSimulatorImpl::ScheduleWithContext (uint32_t context, Time const &delay, EventImpl *impl)
 {
-  NS_LOG_FUNCTION (time << impl);
+  NS_LOG_FUNCTION (this << context << delay << impl);
 
   {
     CriticalSection cs (m_mutex);
     uint64_t ts;
 
-    ts = m_currentTs + time.GetTimeStep ();
+    if (SystemThread::Equals (m_main))
+      {
+        ts = m_currentTs + delay.GetTimeStep ();
+      }
+    else
+      {
+        //
+        // If the simulator is running, we're pacing and have a meaningful 
+        // realtime clock.  If we're not, then m_currentTs is where we stopped.
+        // 
+        ts = m_running ? m_synchronizer->GetCurrentRealtime () : m_currentTs;
+        ts += delay.GetTimeStep ();
+      }
+
     NS_ASSERT_MSG (ts >= m_currentTs, "RealtimeSimulatorImpl::ScheduleRealtime(): schedule for time < m_currentTs");
     Scheduler::Event ev;
     ev.impl = impl;
@@ -598,7 +577,7 @@ RealtimeSimulatorImpl::ScheduleWithContext (uint32_t context, Time const &time,
 EventId
 RealtimeSimulatorImpl::ScheduleNow (EventImpl *impl)
 {
-  NS_LOG_FUNCTION_NOARGS ();
+  NS_LOG_FUNCTION (this << impl);
   Scheduler::Event ev;
   {
     CriticalSection cs (m_mutex);
@@ -628,7 +607,7 @@ RealtimeSimulatorImpl::Now (void) const
 void
 RealtimeSimulatorImpl::ScheduleRealtimeWithContext (uint32_t context, Time const &time, EventImpl *impl)
 {
-  NS_LOG_FUNCTION (context << time << impl);
+  NS_LOG_FUNCTION (this << context << time << impl);
 
   {
     CriticalSection cs (m_mutex);
@@ -649,14 +628,14 @@ RealtimeSimulatorImpl::ScheduleRealtimeWithContext (uint32_t context, Time const
 void
 RealtimeSimulatorImpl::ScheduleRealtime (Time const &time, EventImpl *impl)
 {
-  NS_LOG_FUNCTION (time << impl);
+  NS_LOG_FUNCTION (this << time << impl);
   ScheduleRealtimeWithContext (GetContext (), time, impl);
 }
 
 void
 RealtimeSimulatorImpl::ScheduleRealtimeNowWithContext (uint32_t context, EventImpl *impl)
 {
-  NS_LOG_FUNCTION (context << impl);
+  NS_LOG_FUNCTION (this << context << impl);
   {
     CriticalSection cs (m_mutex);
 
@@ -682,7 +661,7 @@ RealtimeSimulatorImpl::ScheduleRealtimeNowWithContext (uint32_t context, EventIm
 void
 RealtimeSimulatorImpl::ScheduleRealtimeNow (EventImpl *impl)
 {
-  NS_LOG_FUNCTION (impl);
+  NS_LOG_FUNCTION (this << impl);
   ScheduleRealtimeNowWithContext (GetContext (), impl);
 }
 
@@ -695,7 +674,7 @@ RealtimeSimulatorImpl::RealtimeNow (void) const
 EventId
 RealtimeSimulatorImpl::ScheduleDestroy (EventImpl *impl)
 {
-  NS_LOG_FUNCTION_NOARGS ();
+  NS_LOG_FUNCTION (this << impl);
 
   EventId id;
   {
@@ -778,12 +757,12 @@ RealtimeSimulatorImpl::Cancel (const EventId &id)
 }
 
 bool
-RealtimeSimulatorImpl::IsExpired (const EventId &ev) const
+RealtimeSimulatorImpl::IsExpired (const EventId &id) const
 {
-  if (ev.GetUid () == 2)
+  if (id.GetUid () == 2)
     {
-      if (ev.PeekEventImpl () == 0 ||
-          ev.PeekEventImpl ()->IsCancelled ())
+      if (id.PeekEventImpl () == 0 ||
+          id.PeekEventImpl ()->IsCancelled ())
         {
           return true;
         }
@@ -791,7 +770,7 @@ RealtimeSimulatorImpl::IsExpired (const EventId &ev) const
       for (DestroyEvents::const_iterator i = m_destroyEvents.begin (); 
            i != m_destroyEvents.end (); i++)
         {
-          if (*i == ev)
+          if (*i == id)
             {
               return false;
             }
@@ -807,10 +786,10 @@ RealtimeSimulatorImpl::IsExpired (const EventId &ev) const
   //
   // The same is true for the next line involving the m_currentUid.
   //
-  if (ev.PeekEventImpl () == 0 ||
-      ev.GetTs () < m_currentTs ||
-      (ev.GetTs () == m_currentTs && ev.GetUid () <= m_currentUid) ||
-      ev.PeekEventImpl ()->IsCancelled ()) 
+  if (id.PeekEventImpl () == 0 ||
+      id.GetTs () < m_currentTs ||
+      (id.GetTs () == m_currentTs && id.GetUid () <= m_currentUid) ||
+      id.PeekEventImpl ()->IsCancelled ()) 
     {
       return true;
     }
@@ -823,8 +802,8 @@ RealtimeSimulatorImpl::IsExpired (const EventId &ev) const
 Time 
 RealtimeSimulatorImpl::GetMaximumSimulationTime (void) const
 {
-  // XXX: I am fairly certain other compilers use other non-standard
-  // post-fixes to indicate 64 bit constants.
+  /// \todo I am fairly certain other compilers use other non-standard
+  /// post-fixes to indicate 64 bit constants.
   return TimeStep (0x7fffffffffffffffLL);
 }
 
@@ -844,28 +823,28 @@ RealtimeSimulatorImpl::GetContext (void) const
 void 
 RealtimeSimulatorImpl::SetSynchronizationMode (enum SynchronizationMode mode)
 {
-  NS_LOG_FUNCTION (mode);
+  NS_LOG_FUNCTION (this << mode);
   m_synchronizationMode = mode;
 }
 
 RealtimeSimulatorImpl::SynchronizationMode
 RealtimeSimulatorImpl::GetSynchronizationMode (void) const
 {
-  NS_LOG_FUNCTION_NOARGS ();
+  NS_LOG_FUNCTION (this);
   return m_synchronizationMode;
 }
 
 void 
 RealtimeSimulatorImpl::SetHardLimit (Time limit)
 {
-  NS_LOG_FUNCTION (limit);
+  NS_LOG_FUNCTION (this << limit);
   m_hardLimit = limit;
 }
 
 Time
 RealtimeSimulatorImpl::GetHardLimit (void) const
 {
-  NS_LOG_FUNCTION_NOARGS ();
+  NS_LOG_FUNCTION (this);
   return m_hardLimit;
 }
 
