@@ -42,12 +42,11 @@
 
 #include "sim_errno.h"
 
-NS_LOG_COMPONENT_DEFINE ("NscTcpSocketImpl");
-
-using namespace std;
 
 namespace ns3 {
 
+NS_LOG_COMPONENT_DEFINE ("NscTcpSocketImpl");
+
 NS_OBJECT_ENSURE_REGISTERED (NscTcpSocketImpl);
 
 TypeId
@@ -55,9 +54,19 @@ NscTcpSocketImpl::GetTypeId ()
 {
   static TypeId tid = TypeId ("ns3::NscTcpSocketImpl")
     .SetParent<TcpSocket> ()
+    .SetGroupName ("Internet")
     .AddTraceSource ("CongestionWindow",
                      "The TCP connection's congestion window",
-                     MakeTraceSourceAccessor (&NscTcpSocketImpl::m_cWnd))
+                     MakeTraceSourceAccessor (&NscTcpSocketImpl::m_cWnd),
+                     "ns3::TracedValueCallback::Uint32")
+    .AddTraceSource ("SlowStartThreshold",
+                     "TCP slow start threshold (bytes)",
+                     MakeTraceSourceAccessor (&NscTcpSocketImpl::m_ssThresh),
+                     "ns3::TracedValueCallback::Uint32")
+    .AddTraceSource ("State",
+                     "TCP state",
+                     MakeTraceSourceAccessor (&NscTcpSocketImpl::m_state),
+                     "ns3::TcpStatesTracedValueCallback")
   ;
   return tid;
 }
@@ -107,9 +116,10 @@ NscTcpSocketImpl::NscTcpSocketImpl(const NscTcpSocketImpl& sock)
     m_cWnd (sock.m_cWnd),
     m_ssThresh (sock.m_ssThresh),
     m_initialCWnd (sock.m_initialCWnd),
+    m_initialSsThresh (sock.m_initialSsThresh),
     m_lastMeasuredRtt (Seconds (0.0)),
     m_cnTimeout (sock.m_cnTimeout),
-    m_cnCount (sock.m_cnCount),
+    m_synRetries (sock.m_synRetries),
     m_rxAvailable (0),
     m_nscTcpSocket (0),
     m_sndBufSize (sock.m_sndBufSize)
@@ -153,6 +163,7 @@ NscTcpSocketImpl::SetNode (Ptr<Node> node)
   m_node = node;
   // Initialize some variables 
   m_cWnd = m_initialCWnd * m_segmentSize;
+  m_ssThresh = m_initialSsThresh;
   m_rxWindowSize = m_advertisedWindowSize;
 }
 
@@ -214,6 +225,13 @@ NscTcpSocketImpl::Bind (void)
   m_endPoint = m_tcp->Allocate ();
   return FinishBind ();
 }
+int
+NscTcpSocketImpl::Bind6 ()
+{
+  NS_LOG_LOGIC ("NscTcpSocketImpl: ERROR_AFNOSUPPORT - Bind6 not supported");
+  m_errno = ERROR_AFNOSUPPORT;
+  return (-1);
+}
 int 
 NscTcpSocketImpl::Bind (const Address &address)
 {
@@ -449,9 +467,6 @@ NscTcpSocketImpl::RecvFrom (uint32_t maxSize, uint32_t flags,
       bool found;
       found = packet->PeekPacketTag (tag);
       NS_ASSERT (found);
-      // cast found to void, to suppress 'found' set but not used
-      // compiler warning in optimized builds
-      (void) found;
       fromAddress = tag.GetAddress ();
     }
   return packet;
@@ -465,6 +480,21 @@ NscTcpSocketImpl::GetSockName (Address &address) const
   return 0;
 }
 
+int
+NscTcpSocketImpl::GetPeerName (Address &address) const
+{
+  NS_LOG_FUNCTION (this << address);
+
+  if (!m_endPoint)
+    {
+      m_errno = ERROR_NOTCONN;
+      return -1;
+    }
+  address = InetSocketAddress (m_endPoint->GetPeerAddress (),
+                               m_endPoint->GetPeerPort ());
+  return 0;
+}
+
 uint32_t
 NscTcpSocketImpl::GetRxAvailable (void) const
 {
@@ -493,7 +523,7 @@ void NscTcpSocketImpl::CompleteFork (void)
 
   if (0 == m_nscTcpSocket->getpeername ((struct sockaddr*) &sin, &sin_len)) {
       m_remotePort = ntohs (sin.sin_port);
-      m_remoteAddress = m_remoteAddress.Deserialize ((const uint8_t*) &sin.sin_addr);
+      m_remoteAddress = Ipv4Address::Deserialize ((const uint8_t*) &sin.sin_addr);
       m_peerAddress = InetSocketAddress (m_remoteAddress, m_remotePort);
     }
 
@@ -506,7 +536,7 @@ void NscTcpSocketImpl::CompleteFork (void)
   sin_len = sizeof(sin);
 
   if (0 == m_nscTcpSocket->getsockname ((struct sockaddr *) &sin, &sin_len))
-    m_localAddress = m_localAddress.Deserialize ((const uint8_t*) &sin.sin_addr);
+    m_localAddress = Ipv4Address::Deserialize ((const uint8_t*) &sin.sin_addr);
 
   NS_LOG_LOGIC ("NscTcpSocketImpl " << this << " accepted connection from " 
                                     << m_remoteAddress << ":" << m_remotePort
@@ -525,7 +555,7 @@ void NscTcpSocketImpl::ConnectionSucceeded ()
   struct sockaddr_in sin;
   size_t sin_len = sizeof(sin);
   if (0 == m_nscTcpSocket->getsockname ((struct sockaddr *) &sin, &sin_len)) {
-      m_localAddress = m_localAddress.Deserialize ((const uint8_t*)&sin.sin_addr);
+      m_localAddress = Ipv4Address::Deserialize ((const uint8_t*)&sin.sin_addr);
       m_localPort = ntohs (sin.sin_port);
     }
 
@@ -727,15 +757,15 @@ NscTcpSocketImpl::GetAdvWin (void) const
 }
 
 void
-NscTcpSocketImpl::SetSSThresh (uint32_t threshold)
+NscTcpSocketImpl::SetInitialSSThresh (uint32_t threshold)
 {
-  m_ssThresh = threshold;
+  m_initialSsThresh = threshold;
 }
 
 uint32_t
-NscTcpSocketImpl::GetSSThresh (void) const
+NscTcpSocketImpl::GetInitialSSThresh (void) const
 {
-  return m_ssThresh;
+  return m_initialSsThresh;
 }
 
 void
@@ -763,15 +793,15 @@ NscTcpSocketImpl::GetConnTimeout (void) const
 }
 
 void 
-NscTcpSocketImpl::SetConnCount (uint32_t count)
+NscTcpSocketImpl::SetSynRetries (uint32_t count)
 {
-  m_cnCount = count;
+  m_synRetries = count;
 }
 
 uint32_t 
-NscTcpSocketImpl::GetConnCount (void) const
+NscTcpSocketImpl::GetSynRetries (void) const
 {
-  return m_cnCount;
+  return m_synRetries;
 }
 
 void 
@@ -780,6 +810,20 @@ NscTcpSocketImpl::SetDelAckTimeout (Time timeout)
   m_delAckTimeout = timeout;
 }
 
+void
+NscTcpSocketImpl::SetDataRetries (uint32_t retries)
+{
+  NS_LOG_FUNCTION (this << retries);
+  m_dataRetries = retries;
+}
+
+uint32_t
+NscTcpSocketImpl::GetDataRetries (void) const
+{
+  NS_LOG_FUNCTION (this);
+  return m_dataRetries;
+}
+
 Time
 NscTcpSocketImpl::GetDelAckTimeout (void) const
 {
@@ -840,7 +884,7 @@ NscTcpSocketImpl::GetNativeNs3Errno (int error) const
     case NSC_EAGAIN: return ERROR_AGAIN;
     case NSC_EISCONN:   // fallthrough
     case NSC_EALREADY: return ERROR_ISCONN;
-    case NSC_ECONNREFUSED: return ERROR_NOROUTETOHOST;   // XXX, better mapping?
+    case NSC_ECONNREFUSED: return ERROR_NOROUTETOHOST;   /// \todo better mapping?
     case NSC_ECONNRESET:   // for no, all of these fall through
     case NSC_EHOSTDOWN:
     case NSC_ENETUNREACH:
@@ -848,7 +892,7 @@ NscTcpSocketImpl::GetNativeNs3Errno (int error) const
     case NSC_EMSGSIZE: return ERROR_MSGSIZE;
     case NSC_ENOTCONN: return ERROR_NOTCONN;
     case NSC_ESHUTDOWN: return ERROR_SHUTDOWN;
-    case NSC_ETIMEDOUT: return ERROR_NOTCONN;   // XXX - this mapping isn't correct
+    case NSC_ETIMEDOUT: return ERROR_NOTCONN;   /// \todo this mapping isn't correct
     case NSC_ENOTDIR:   // used by eg. sysctl(2). Shouldn't happen normally,
     // but is triggered by e.g. show_config().
     case NSC_EUNKNOWN: return ERROR_INVAL;   // Catches stacks that 'return -1' without real mapping
