@@ -18,23 +18,24 @@
  * Author: Nicola Baldo <nbaldo@cttc.es>
  */
 
-#include <ns3/waveform-generator.h>
 #include <ns3/object-factory.h>
 #include <ns3/log.h>
-#include <math.h>
+#include <cmath>
 #include <ns3/simulator.h>
 #include <ns3/trace-source-accessor.h>
 #include <ns3/packet-burst.h>
 #include <ns3/callback.h>
+#include <ns3/antenna-model.h>
+
 #include "half-duplex-ideal-phy.h"
 #include "half-duplex-ideal-phy-signal-parameters.h"
 #include "spectrum-error-model.h"
 
 
-NS_LOG_COMPONENT_DEFINE ("HalfDuplexIdealPhy");
-
 namespace ns3 {
 
+NS_LOG_COMPONENT_DEFINE ("HalfDuplexIdealPhy");
+
 NS_OBJECT_ENSURE_REGISTERED (HalfDuplexIdealPhy);
 
 HalfDuplexIdealPhy::HalfDuplexIdealPhy ()
@@ -70,6 +71,12 @@ HalfDuplexIdealPhy::DoDispose ()
   SpectrumPhy::DoDispose ();
 }
 
+/**
+ * \brief Output stream operator
+ * \param os output stream
+ * \param s the state to print
+ * \return an output stream
+ */
 std::ostream& operator<< (std::ostream& os, HalfDuplexIdealPhy::State s)
 {
   switch (s)
@@ -105,22 +112,28 @@ HalfDuplexIdealPhy::GetTypeId (void)
                    MakeDataRateChecker ())
     .AddTraceSource ("TxStart",
                      "Trace fired when a new transmission is started",
-                     MakeTraceSourceAccessor (&HalfDuplexIdealPhy::m_phyTxStartTrace))
+                     MakeTraceSourceAccessor (&HalfDuplexIdealPhy::m_phyTxStartTrace),
+                     "ns3::Packet::TraceCallback")
     .AddTraceSource ("TxEnd",
                      "Trace fired when a previosuly started transmission is finished",
-                     MakeTraceSourceAccessor (&HalfDuplexIdealPhy::m_phyTxEndTrace))
+                     MakeTraceSourceAccessor (&HalfDuplexIdealPhy::m_phyTxEndTrace),
+                     "ns3::Packet::TraceCallback")
     .AddTraceSource ("RxStart",
                      "Trace fired when the start of a signal is detected",
-                     MakeTraceSourceAccessor (&HalfDuplexIdealPhy::m_phyRxStartTrace))
+                     MakeTraceSourceAccessor (&HalfDuplexIdealPhy::m_phyRxStartTrace),
+                     "ns3::Packet::TraceCallback")
     .AddTraceSource ("RxAbort",
                      "Trace fired when a previously started RX is aborted before time",
-                     MakeTraceSourceAccessor (&HalfDuplexIdealPhy::m_phyRxAbortTrace))
+                     MakeTraceSourceAccessor (&HalfDuplexIdealPhy::m_phyRxAbortTrace),
+                     "ns3::Packet::TraceCallback")
     .AddTraceSource ("RxEndOk",
                      "Trace fired when a previosuly started RX terminates successfully",
-                     MakeTraceSourceAccessor (&HalfDuplexIdealPhy::m_phyRxEndOkTrace))
+                     MakeTraceSourceAccessor (&HalfDuplexIdealPhy::m_phyRxEndOkTrace),
+                     "ns3::Packet::TraceCallback")
     .AddTraceSource ("RxEndError",
                      "Trace fired when a previosuly started RX terminates with an error (packet is corrupted)",
-                     MakeTraceSourceAccessor (&HalfDuplexIdealPhy::m_phyRxEndErrorTrace))
+                     MakeTraceSourceAccessor (&HalfDuplexIdealPhy::m_phyRxEndErrorTrace),
+                     "ns3::Packet::TraceCallback")
   ;
   return tid;
 }
@@ -128,7 +141,7 @@ HalfDuplexIdealPhy::GetTypeId (void)
 
 
 Ptr<NetDevice>
-HalfDuplexIdealPhy::GetDevice ()
+HalfDuplexIdealPhy::GetDevice () const
 {
   NS_LOG_FUNCTION (this);
   return m_netDevice;
@@ -241,6 +254,20 @@ HalfDuplexIdealPhy::SetGenericPhyRxEndOkCallback (GenericPhyRxEndOkCallback c)
   m_phyMacRxEndOkCallback = c;
 }
 
+Ptr<AntennaModel>
+HalfDuplexIdealPhy::GetRxAntenna ()
+{
+  NS_LOG_FUNCTION (this);
+  return m_antenna;
+}
+
+void
+HalfDuplexIdealPhy::SetAntenna (Ptr<AntennaModel> a)
+{
+  NS_LOG_FUNCTION (this << a);
+  m_antenna = a;
+}
+
 void
 HalfDuplexIdealPhy::ChangeState (State newState)
 {
@@ -267,15 +294,16 @@ HalfDuplexIdealPhy::StartTx (Ptr<Packet> p)
         m_txPacket = p;
         ChangeState (TX);
         Ptr<HalfDuplexIdealPhySignalParameters> txParams = Create<HalfDuplexIdealPhySignalParameters> ();
-        double txTimeSeconds = m_rate.CalculateTxTime (p->GetSize ());
-        txParams->duration = Seconds (txTimeSeconds);
+        Time txTimeSeconds = m_rate.CalculateBytesTxTime (p->GetSize ());
+        txParams->duration = txTimeSeconds;
         txParams->txPhy = GetObject<SpectrumPhy> ();
+        txParams->txAntenna = m_antenna;
         txParams->psd = m_txPsd;
         txParams->data = m_txPacket;
 
-        NS_LOG_LOGIC (this << " tx power: " << 10 * log10 (Integral (*(txParams->psd))) + 30 << " dBm");
+        NS_LOG_LOGIC (this << " tx power: " << 10 * std::log10 (Integral (*(txParams->psd))) + 30 << " dBm");
         m_channel->StartTx (txParams);
-        Simulator::Schedule (Seconds (txTimeSeconds), &HalfDuplexIdealPhy::EndTx, this);
+        Simulator::Schedule (txTimeSeconds, &HalfDuplexIdealPhy::EndTx, this);
       }
       break;
 
@@ -313,7 +341,7 @@ HalfDuplexIdealPhy::StartRx (Ptr<SpectrumSignalParameters> spectrumParams)
 {
   NS_LOG_FUNCTION (this << spectrumParams);
   NS_LOG_LOGIC (this << " state: " << m_state);
-  NS_LOG_LOGIC (this << " rx power: " << 10 * log10 (Integral (*(spectrumParams->psd))) + 30 << " dBm");
+  NS_LOG_LOGIC (this << " rx power: " << 10 * std::log10 (Integral (*(spectrumParams->psd))) + 30 << " dBm");
 
   // interference will happen regardless of the state of the receiver
   m_interference.AddSignal (spectrumParams->psd, spectrumParams->duration);
